﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;

namespace SkinFileUpgrader
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length < 1)
            {
                Console.WriteLine("Drag and drop a single skin file (dashboard.css, dashboard.js or dashboard.html) to this executable to upgrade it to 3.0.0+.");
                ExitPrompt();
            }

            string fileName = args[0];
            string extension = Path.GetExtension(fileName);
            string inputText = File.ReadAllText(fileName, Encoding.UTF8);
            string outputText = string.Empty;
            
            switch (extension)
            {
                case ".less":
                case ".css":
                {
                    outputText = Upgrade(inputText, GetReplaces(OldCss, NewCss));
                    break;
                }
                case ".html":
                {
                    outputText = Upgrade(inputText, GetReplaces(OldHtml, NewHtml));
                    break;
                }
                case ".js":
                {
                    outputText = Upgrade(inputText, GetReplaces(OldJs, NewJs));
                    outputText = (new Regex("(\\s*)(return data;)"))
                        .Replace(outputText, "$1data.hasJob = data.trailer.attached;\r\n$1$2");
                    break;
                }
                default:
                {
                    Console.WriteLine("{0} extension is not supported.", extension);
                    ExitPrompt();
                    break;
                }
            }
            
            string backupFileName = Path.ChangeExtension(fileName, extension + ".old");
            File.Move(fileName, backupFileName);
            File.WriteAllText(fileName, outputText, Encoding.UTF8);

            Console.BackgroundColor = ConsoleColor.Black;
            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("Done!");
            Thread.Sleep(TimeSpan.FromSeconds(1));
        }

        static void ExitPrompt()
        {
            Console.WriteLine("Press any key to exit...");
            Console.ReadKey();
            Environment.Exit(0);
        }

        static string OldHtml(string name)
        {
            return "(['\" \\n])(" + name + ")([\\.'\" \\r\\n])";
        }

        static string NewHtml(string name)
        {
            return "$1" + name + "$3";
        }

        static string OldCss(string name)
        {
            return "([\\.])(" + name + ")([\\)\\.'\",; \\r\\n\\[{])";
        }

        static string NewCss(string name)
        {
            return "$1" + name + "$3";
        }

        static string OldJs(string name)
        {
            return "data\\.(" + name + ")([\\*\\/\\-\\+\\)'\",; \r\n\\[])";
        }

        static string NewJs(string name)
        {
            return "data." + name.Replace('-', '.') + "$2";
        }
        
        static Dictionary<string, string> GetReplaces(Func<string,string> oldFunc, Func<string,string> newFunc)
        {
            return new Dictionary<string, string>
            {
                {"connected", "game-connected"},
                {"gameTime", "game-time"},
                {"gamePaused", "game-paused"},
                {"telemetryPluginVersion", "game-telemetryPluginVersion"},
                {"gameVersion", "game-version"},
                {"truckMakeId", "truck-id"},
                {"truckModel", "truck-model"},
                {"truckMake", "truck-make"},
                {"truckSpeedRounded", "truck-speedRounded"},
                {"truckSpeed", "truck-speed"},
                {"engineRpmMax", "truck-engineRpmMax"},
                {"engineRpm", "truck-engineRpm"},
                {"truckOdometer", "truck-odometer"},
                {"cruiseControlSpeedRounded", "truck-cruiseControlSpeedRounded"},
                {"cruiseControlSpeed", "truck-cruiseControlSpeed"},
                {"fuelCapacity", "truck-fuelCapacity"},
                {"fuelWarningFactor", "truck-fuelWarningFactor"},
                {"wipersOn", "truck-wipersOn"},
                {"truckSpeedMph", "truck-speedMph"},
                {"blinkerLeftOn", "truck-blinkerLeftOn"},
                {"blinkerRightOn", "truck-blinkerRightOn"},
                {"cruiseControlOn", "truck-cruiseControlOn"},
                {"lightsBeamHighOn", "truck-lightsBeamHighOn"},
                {"lightsBeamLowOn", "truck-lightsBeamLowOn"},
                {"lightsParkingOn", "truck-lightsParkingOn"},
                {"lightsAuxFrontOn", "truck-lightsAuxFrontOn"},
                {"lightsAuxRoofOn", "truck-lightsAuxRoofOn"},
                {"lightsBeaconOn", "truck-lightsBeaconOn"},
                {"lightsBrakeOn", "truck-lightsBrakeOn"},
                {"lightsReverseOn", "truck-lightsReverseOn"},
                {"trailerAttached", "trailer-attached"},
                {"batteryVoltageWarning", "truck-batteryVoltageWarningOn"},
                {"adblueWarning", "truck-adblueWarningOn"},
                {"oilPressureWarning", "truck-oilPressureWarningOn"},
                {"waterTemperatureWarning", "truck-waterTemperatureWarningOn"},
                {"brakeTemperature", "truck-brakeTemperature"},
                {"fuelWarning", "truck-fuelWarningOn"},
                {"fuelAverageConsumption", "truck-fuelAverageConsumption"},
                {"oilPressure", "truck-oilPressure"},
                {"waterTemperature", "truck-waterTemperature"},
                {"lightsDashboard", "truck-lightsDashboard"},
                {"wearEngine", "truck-wearEngine"},
                {"wearTransmission", "truck-wearTransmission"},
                {"wearCabin", "truck-wearCabin"},
                {"wearChassis", "truck-wearChassis"},
                {"wearWheels", "truck-wearWheels"},
                {"trailerMass", "trailer-mass"},
                {"trailerName", "trailer-name"},
                {"trailerId", "trailer-id"},
                {"sourceCity", "job-sourceCity"},
                {"sourceCompany", "job-sourceCompany"},
                {"destinationCity", "job-destinationCity"},
                {"destinationCompany", "job-destinationCompany"},
                {"jobRemainingTime", "job-remainingTime"},
                {"jobDeadlineTime", "job-deadlineTime"},
                {"jobIncome", "job-income"},
                {"wearTrailer", "trailer-wear"},
                {"electricOn", "truck-electricOn"},
                {"engineOn", "truck-engineOn"},
                {"blinkerLeftActive", "truck-blinkerLeftActive"},
                {"blinkerRightActive", "truck-blinkerRightActive"},
                {"parkBrakeOn", "truck-parkBrakeOn"},
                {"motorBrakeOn", "truck-motorBrakeOn"},
                {"airPressureWarning", "truck-airPressureWarningOn"},
                {"airPressure", "truck-airPressure"},
                {"retarderBrake", "truck-retarderBrake"},
                {"oilTemperature", "truck-oilTemperature"},
                {"batteryVoltage", "truck-batteryVoltage"},
                {"airPressureEmergency", "truck-airPressureEmergencyOn"},
                {"fuel", "truck-fuel"},
                {"gears", "truck-forwardGears"},
                {"gear", "truck-gear"},
            }
            .OrderByDescending(kv => kv.Key.Length)
            .ToDictionary(kv => oldFunc(kv.Key), kv => newFunc(kv.Value));
        }

        static string Upgrade(string input, Dictionary<string, string> replaces)
        {
            return replaces.Aggregate(input, (current, pair) => 
                (new Regex(pair.Key)).Replace(current, pair.Value));
        }
    }
}
